#-*-coding:utf-8-*-
"""
    Teste les fonctions du module amon.era
"""
import os
from amon.era import postes
num_zone = 0
eth = "eth1"

dest_rule = """/sbin/iptables -I FORWARD -i %s -d %s -j REJECT\n/sbin/iptables -I INPUT -i %s -s %s -j REJECT\n"""

poste1 = '1.1.1.1'
hdeb1 = '10:00'
hfin1 = '15:00'
jdeb1 = 'lundi'
jfin1 = 'jeudi'
# FIXME : normalement l'option -o est calculée
poste1_forbid_rule = """/sbin/iptables -I INPUT -p tcp --dport 3128 -i %s -s %s -m time --timestart %s --timestop %s --weekdays Mon,Tue,Wed,Thu -j REJECT\n/sbin/iptables -I FORWARD -m multiport -p tcp --dports 80,443,3128,8080 -i %s -o eth0 -s %s -m time --timestart %s --timestop %s --weekdays Mon,Tue,Wed,Thu -j REJECT\n""" % (eth, poste1, hdeb1, hfin1, eth, poste1, hdeb1, hfin1)
poste1_dest_rule = dest_rule % (eth, poste1, eth, poste1)

poste2 = '1.1.1.2'
hdeb2 = '15:00'
hfin2 = '19:00'
jdeb2 = 'Thu'
jfin2 = 'Mon'
poste2_dest_rule = dest_rule % (eth, poste2, eth, poste2)


def setup_module(module):
    """ initialisation """
    if not os.path.isdir("/usr/share/eole/tests/datas/"):
        os.system("mkdir /usr/share/eole/tests/datas")
    postes.f_poste_name = {'all':'/usr/share/eole/tests/datas/test_all%d.txt',
                           'web':'/usr/share/eole/tests/datas/test_web%d.txt',
                           'destination':'/usr/share/eole/tests/datas/test_dest%d.txt'}

def test_get_interface():
    """ teste la détection d'interface """
    assert 'eth1' in postes.get_interface(num_zone)

def test_web_interdiction():
    """ teste l'interdiction web pour un ou plusieurs postes
    """
    p1 = dict(poste=poste1, hdeb=hdeb1, hfin=hfin1, jdeb=jdeb1, jfin=jfin1, interface=eth)
    p2 = dict(poste=poste2, hdeb=hdeb2, hfin=hfin2, jdeb=jdeb2, jfin=jfin2, interface=eth)
    assert postes.save_forbidden_poste(0, 'web', [p1, p2])
    assert postes.load_forbidden_poste(0, 'web') == [p1, p2]
    assert postes.save_forbidden_poste(0, 'web', [p1])
    assert postes.generate_postes_rules()[0] == poste1_forbid_rule
    postes.save_forbidden_poste(0, 'web', [])
    assert postes.load_forbidden_poste(0, 'web') == []

def test_all_interdiction():
    p1 = dict(poste=poste1, interface=eth)
    p2 = dict(poste=poste2)
    postes.save_forbidden_poste(0, 'all', [p1, p2])
    assert postes.load_forbidden_poste(0, 'all') == [p1, p2]
    postes.save_forbidden_poste(0, 'all', [])
    assert postes.load_forbidden_poste(0, 'all') == []

def test_destination():
    dest1 = dict(poste=poste1, interface=eth)
    dest2 = dict(poste=poste2)
    postes.save_forbidden_poste(0, 'destination', [dest1, dest2])
    assert postes.load_forbidden_poste(0, 'destination') == [dest1, dest2]
    assert postes.generate_destinations_rules() == [poste1_dest_rule, poste2_dest_rule]
    postes.save_forbidden_poste(0, 'destination', [])
    assert postes.load_forbidden_poste(0, 'destination') == []

def teardown_module(module):
    for _file in postes.f_poste_name.values():
        if os.path.isfile(_file%0):
            os.unlink(_file%0)
