# -*- coding: utf-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# Génère les règles ipset pour les groupes de machine
#
###########################################################################

from amon.ipset import group_manager
from amon.ipset.parser import Parser
from amon.backend import command_statusoutput, get_filter_zones

# nom du set de services utilise pour toutes les restrictions (avec tous les groupes de machine)
SERVICE_SET_NAME = 'web_services'

# syntaxe des règles ipset liées aux groupes à appliquer
# creation de groupe/ajout d'element/binding du groupe d'ip avec un groupe de service
GROUP = "/usr/sbin/ipset -N %(name)s ipmap --from %(ip_from)s --to %(ip_to)s"
GROUP_ADD = "/usr/sbin/ipset -A %s %s"
BIND = "/usr/sbin/ipset -B %s :default: -b %s"
GROUP_DELETE = "/usr/sbin/ipset -X %s"

# syntaxe des règles ipset des groupes de service
SERVICE = "ipset -N %s portmap --from 0 --to 65000"
SERVICE_ADD = "ipset -A %s 3128"

class RuleGenerator:
    """ crée les groupes de machine au niveau ipset """

    def __init__(self):
        """ parser: outil de parsing de la commande ipset --save
            existing_ipsets: ipset déjà créé
        """
        self.parser = Parser()
        code, values = command_statusoutput('/usr/sbin/ipset --save')
        if code:
            raise Exception, "Erreur : erreur lors de l'éxécution ipset --save"
        parsed_lines = self.parse_ipsets(values)
        self.existing_ipsets = [group['name'] for group in parsed_lines['create']]

    def create_ip_sets(self):
        """ crée et supprime les groupes d'ips configurés depuis l'interface ead2
            niveau ipset
            lien entre le fichier de configuration édité depuis le group_manager et ipset
        """
        ## creation du set de service si ce n'est pas fait
        if SERVICE_SET_NAME not in self.existing_ipsets:
            code, retour = command_statusoutput(SERVICE%SERVICE_SET_NAME)
            if code:
                raise Exception, "Erreur : erreur lors de la création du set de service %s : %s"%(SERVICE_SET_NAME, retour)
            code, retour = command_statusoutput(SERVICE_ADD%SERVICE_SET_NAME)
            if code:
                raise Exception, "Erreur : erreur lors de l'ajout du port 3128 au set de service : %s"%retour
        ## creation des groupes de machine
        for zone in get_filter_zones().keys():
            for group in [gr for gr in group_manager.get_all_groups(zone) if gr['name'] not in self.existing_ipsets]:
                code, retour = command_statusoutput(regles_create_ip_set(group))
                if code:
                    raise Exception, "Erreur : erreur lors de la création du set d'ip %s : %s"%(group['name'], retour)
                for rule in regles_populate_ip_set(group):
                    code, retour = command_statusoutput(rule)
                    if code:
                        raise Exception, "Erreur : erreur lors de l'ajout d'ip dans le groupe %s: éxécution de %s : %s"%(group['name'], rule, retour)
                code, retour = command_statusoutput(regles_bind_ip_set(group))
                if code:
                    raise Exception, "Erreur : erreur lors du binding du set d'ip %s set de service web : %s"%(group['name'], retour)

        ## suppression des groupes
        for group in self.existing_ipsets:
            if group not in [gr['name'] for gr in get_all_groups()] and group != SERVICE_SET_NAME:
                code, retour = command_statusoutput(regles_delete_ip_set(group))
                if code:
                    raise Exception, "Erreur : erreur lors de la suppression du set d'ip %s : %s"%(group, retour)

    def parse_ipsets(self, datas):
        """ parse le résultat de ipset --save
            et renvoie un dico décrivant les instructions effectuées
            :create: les groupes créés
            :add: membres ajoutés au groupe
                    (ne sert à rien dans l'utilisation faite actuellement d'ipset)
            :bind: lien entre les sets de machine et les sets de services(ports)
                    (ne sert à rien dans l'utilisation faite actuellement d'ipset)
        """
        parsed_lines = {'create':[], 'add':[], 'bind':[]}
        lines = datas.splitlines()
        for line in lines:
            if line.startswith('-'):
                parsed_line = self.parser.parse_ipset_line([line])
                parsed_lines[parsed_line['instruction']].append(parsed_line)
        return parsed_lines

# fonctions renvoyant les règles ipsets
def regles_create_ip_set(group):
    """ renvoie la règle pour la création de groupes ipset
        @group: nom du groupe
    """
    #print "On crée le groupe ipset : %s"%group
    return GROUP%(group)

def regles_populate_ip_set(group):
    """ renvoie les règles permettant de peupler un set d'ip
    """
    for ip in get_ips_range(group['ip_from'], group['ip_to']):
        yield GROUP_ADD%(group['name'], ip)

def regles_bind_ip_set(group):
    """ renvoie les règles pour peupler lier un set d'ip avec le set de service web
        @group: nom du groupe
    """
    return BIND%(group['name'], SERVICE_SET_NAME)

def regles_delete_ip_set(group):
    """ renvoie la règle pour la suppression d'un set d'ip
        @group: nom du groupe
    """
    return GROUP_DELETE%group


# fonctions utiles
def get_all_groups():
    groups = []
    for zone in get_filter_zones().keys():
        groups.extend(group_manager.get_all_groups(zone))
    return groups

def get_ips_range(start, end):
    """ renvoie la liste des ips comprises dans la plage start-end
        @start: ip de début de plage
        @end: ip de fin de plage
    """
    group_manager.test_ip_range(start, end)
    start = start.split('.')
    end = end.split('.')
    begin_chain = '.'.join(start[:-1])
    ip_range = []
    for num in range(int(start[-1:][0]), (int(end[-1:][0])+1)):
        ip_range.append("%s.%d"%(begin_chain, num))
    return ip_range
